from wikijspy import Configuration, ApiClient, PagesApi
from wikijspy.types.page_types import PageResponseOutput
import json

with open('examples/settings.json', 'r') as f:
    settings_dict = json.loads(f.read())
    WIKIJS_HOST = settings_dict["hostname"]
    TOKEN = settings_dict["token"]

conf = Configuration(WIKIJS_HOST, TOKEN)

pages_api = PagesApi(ApiClient(conf))

print(pages_api.create(PageResponseOutput({
    "responseResult": [
        "errorCode",
        "slug",
        "message"
    ],
    "page": [
        "id",
        "path"
    ]}),
    content="<a>Test</a>",
    description="Desc",
    editor="markdown",
    isPublished=True,
    isPrivate=True,
    locale="en",
    path="Test",
    tags=[],
    title="Test"
    ))


from wikijspy import Configuration, PagesApi, ApiClient
from wikijspy.types.page_types import PageResponseOutput

import json



with open('examples/settings.json', 'r') as f:
    settings_dict = json.loads(f.read())
    WIKIJS_HOST = settings_dict["hostname"]
    TOKEN = settings_dict["token"]

conf = Configuration(WIKIJS_HOST, TOKEN)

pages_api = PagesApi(ApiClient(conf))

print(pages_api.update(PageResponseOutput({
    "responseResult": [
        "message",
        "errorCode"
    ],
    "page": [
        "title",
        "id"
    ]
}),
    id=10637,
    isPrivate=True,
    isPublished=True
))





class InvalidOutputError(Exception):
    def __init__(self, output: str, caller: str):
        self.output = output
        super().__init__(f"The given output parameter \"{output}\" is invalid for {caller}!")



from typing import Dict, List, Tuple
from wikijspy.types.error import InvalidOutputError

class Output:
    def __init__(self, output: List[str]) -> None:
        for item in output:
            if not item in self._validation_list:
                raise InvalidOutputError(item, self.__class__.__name__)
        self.output = output
    
    def __iter__(self):
        self.i = 0
        self.max = len(self.output)-1
        return self
    
    def __next__(self):
        if self.i <= self.max:
            result = self.i
            self.i += 1
            return self.output[result]
        else:
            raise StopIteration

    def __getitem__(self, item: int):
        return self.output[item]
    
class DictOutput:
    def __init__(self, output: Dict[str, List[str]]):
        if not isinstance(output, dict):
            raise Exception(f"{self.__class__.__name__} needs an dictionary as parameter!")
        for key in output.keys():
            if not key in list(self._validation_list.keys()):
                raise InvalidOutputError(','.join(list(output.keys())), self.__class__.__name__)
        for v_key in self._validation_list.keys():
            for output_key in output.keys():
                if output_key != v_key:
                    continue
                for i in output[output_key]:
                    if not i in self._validation_list[v_key]:
                        raise InvalidOutputError(i, self.__class__.__name__)
        
        for key,val in output.items():
            if not isinstance(val, list):
                raise Exception(f"{self.__class__.__name__} values behind the dict keys must be of type list!")

            for item in val:
                if not item in self._validation_list[key]:
                    raise InvalidOutputError(item, self.__class__.__name__)
        
        self.output = output
        
        self.iter_dict = []
        
        for element in self.output:
            for item in self.output[element]:
                self.iter_dict.append((element, item))
    
    def __iter__(self):
        self.i = 0
        self.max = len(self.iter_dict)-1
        return self
    
    def __next__(self) -> Tuple[str, str]:
        if self.i <= self.max:
            result = self.iter_dict[self.i]
            self.i += 1
            return result
        else:
            raise StopIteration
    
    def __getitem__(self, item: int):
        return self.iter_dict[item]

class ResponseStatusOutput(Output):
    _validation_list = [
        "succeeded",
        "errorCode",
        "slug",
        "message"
    ]

    def __init__(self, output: List[str]) -> None:
        for item in output:
            if not item in self._validation_list:
                raise InvalidOutputError(item, self.__class__.__name__)
        self.output = output

class DefaultResponseOutput(DictOutput):
    _validation_list = {
        "responseResult": ResponseStatusOutput._validation_list
    }
    
    def __init__(self, output: Dict[str, List[str]]):
        if not output.get("responseResult"):
            raise Exception(f"{self.__class__.__name__} does need to have at least one output!")
        
        for item in output:
            if not isinstance(output[item], list):
                raise Exception(f"Value of dict[\"{item}\"] must be of list type!")
            
            if not item in self._validation_list:
                raise InvalidOutputError(val, self.__class__.__name__)
            
            for val in output[item]:
                if not val in ResponseStatusOutput._validation_list:
                    raise InvalidOutputError(val, self.__class__.__name__)
        
        self.output = output
        
        self.iter_dict = []
        
        for element in self.output:
            for item in self.output[element]:
                self.iter_dict.append((element, item))






from enum import Enum
from wikijspy.types.general import Output

class AssetKind(Enum):
    IMAGE = 0
    BINARY = 1
    ALL = 2

class AssetItemOutput(Output):
    _validation_list = [
        "id",
        "filename",
        "ext",
        "kind",
        "mime",
        "fileSize",
        "metadata",
        "createdAt",
        "updatedAt",
        "folder",
        "author"
    ]

class AssetFolderOutput(Output):
    _validation_list = [
        "id",
        "slug",
        "name"
    ]



import json
from wikijspy.api_client import ApiClient
from wikijspy.api.shared import _generate_output_str
from wikijspy.types.authentication_types import AuthenticationStrategyOutput, AuthenticationActiveStrategyOutput
class AuthenticationApi:
    
    def __init__(self, api_client: ApiClient) -> None:
        self.api_client = api_client

    def strategies(self, output: AuthenticationStrategyOutput):
        query = """
        query{
            authentication{
                strategies{
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        return self.api_client.send_request(query)
    
    def activeStrategies(self, output: AuthenticationActiveStrategyOutput, enabledOnly: bool = None):
        query = """
        query($enabledOnly: Boolean){
            authentication{
                activeStrategies(
                    enabledOnly: $enabledOnly
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))
        
        return self.api_client.send_request(query, json.dumps({
            "enabledOnly": enabledOnly
        }))


from typing import Dict, List
from wikijspy.types.error import InvalidOutputError
from wikijspy.types.general import DictOutput, Output, ResponseStatusOutput
from wikijspy.types.page_types import PageOutput

class UserMinimalOutput(Output):
    _validation_list = [
        "id",
        "name",
        "email",
        "providerKey",
        "isSystem",
        "isActive",
        "createdAt",
        "lastLoginAt"
    ]

class UserOutput(Output):
    _validation_list = [
        "id",
        "name",
        "email",
        "providerKey",
        "providerName",
        "providerId",
        "providerIs2FACapable",
        "isSystem",
        "isActive",
        "isVerified",
        "location",
        "jobTitle",
        "timezone",
        "dateFormat",
        "appearance",
        "createdAt",
        "updatedAt",
        "lastLoginAt",
        "tfaIsActive",
        "groups"
    ]

class UserResponseOutput(DictOutput):
    _validation_list = {
        "responseResult": ResponseStatusOutput._validation_list,
        "user": UserOutput._validation_list
    }



from typing import Tuple

def _generate_output_str(output) -> str:
    if type(output[0]) is str:
        output_str: str = ""
        for item in output:
            output_str += item+","
        return output_str
    if isinstance(output[0], Tuple):
        output_str: str = ""
        output_dict: Tuple[str, str] = {}
        
        for item in output:
            if output_dict.get(item[0]) is None:
                output_dict[item[0]] = []
            output_dict[item[0]].append(item[1])
        
        for key,val in output_dict.items():
            if not val:
                continue
            
            output_str += key+'{'
            
            for item in val:
                output_str += item+','
            output_str += '}'
        return output_str



from enum import Enum
from wikijspy.types.general import Output


class AuthenticationStrategyOutput(Output):
    _validation_list = [
        "key",
        "props",
        "title",
        "description",
        "isAvailable",
        "useForm",
        "usernameType",
        "logo",
        "color",
        "website",
        "icon"
    ]

class AuthenticationActiveStrategyOutput(Output):
    _validation_list = [
        "key",
        "strategy",
        "displayName",
        "order",
        "isEnabled",
        "config",
        "selfRegistration",
        "domainWhitelist",
        "autoEnrollGroups"
    ]

class AuthenticationUserErrors(Enum):
    Nothing = 0
    AuthGenericError = 1001
    AuthLoginFailed = 1002
    AuthProviderInvalid = 1003
    AuthAccountAlreadyExists = 1004
    AuthTFAFailed = 1005
    AuthTFAInvalid = 1006
    BruteInstanceIsInvalid = 1007
    BruteTooManyAttempts = 1008
    UserCreationFailed = 1009
    AuthRegistrationDisabled = 1010
    AuthRegistrationDomainUnauthorized = 1011
    InputInvalid = 1012
    AuthAccountBanned = 1013
    AuthAccountNotVerified = 1014
    AuthValidationTokenInvalid = 1015
    UserNotFound = 1016
    UserDeleteForeignConstraint = 1017
    UserDeleteProtected = 1018
    AuthRequired = 1019
    AuthPasswordInvalid = 1020



from wikijspy import ApiClient
from wikijspy.types.assets_types import *
from wikijspy.api.shared import _generate_output_str
from wikijspy.types.general import DefaultResponseOutput
import json


class AssetsApi:
    def __init__(self, api_client: ApiClient) -> None:
        self.api_client = api_client

    def list(self, output: AssetItemOutput, folderId: int, kind: AssetKind):
        query = """
        query($folderId: Int!, $kind: AssetKind!){
            assets{
                list(
                    folderId: $folderId,
                    kind: $kind
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))
        
        return self.api_client.send_request(query, json.dumps({
            "folderId": folderId,
            "kind": kind
        }))
    
    def folders(self, output: AssetFolderOutput, parentFolderId: int):
        query = """
        query($parentFolderId: Int!){
            assets{
                folders(
                    parentFolderId: $parentFolderId
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))
        
        return self.api_client.send_request(query, json.dumps({
            "parentFolderId": parentFolderId
        }))
    
    def createFolder(self, output: DefaultResponseOutput, parentFolderId: int, slug: str, name: str = None):
        query = """
        mutation($parentFolderId: Int!, $slug: String!, $name: String){
            assets{
                createFolder(
                    parentFolderId: $parentFolderId,
                    slug: $slug,
                    name: $name
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        self.api_client.send_request(query, json.dumps({
            "parentFolderId": parentFolderId,
            "slug": slug,
            "name": name
        }))

    def renameAsset(self, output: DefaultResponseOutput, id: int, filename: str):
        query = """
        mutation($id: Int!, $filename: String!){
            assets{
                renameAsset(
                    id: $id,
                    filename: $filename
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        self.api_client.send_request(query, json.dumps({
            "id": id,
            "filename": filename
        }))

    def deleteAsset(self, output: DefaultResponseOutput, id: int):
        query = """
        mutation($id: Int!){
            assets{
                deleteAsset(
                    id: $id
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        self.api_client.send_request(query, json.dumps({
            "id": id
        }))

    def flushTempUploads(self, output: DefaultResponseOutput):
        query = """
        mutation{
            assets{
                flushTempUploads{
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        self.api_client.send_request(query)







from wikijspy import Configuration, ApiClient, PagesApi
from wikijspy.types.page_types import PageOutput, PageResponseOutput
import json

with open('examples/settings.json', 'r') as f:
    settings_dict = json.loads(f.read())
    WIKIJS_HOST = settings_dict["hostname"]
    TOKEN = settings_dict["token"]

conf = Configuration(WIKIJS_HOST, TOKEN)

pages_api = PagesApi(ApiClient(conf))

print(pages_api.single(PageOutput(["content"]), 10637))


import json
from wikijspy.configuration import Configuration
from gql.transport.requests import RequestsHTTPTransport
from gql.client import Client
from gql import gql


class ApiClient:
    def __init__(self, configuration: Configuration) -> None:
        self.client = configuration
        self.hostname = configuration.hostname
        self.token = configuration.token
    
    @property
    def client(self):
        return self.__client
    
    @client.setter
    def client(self, conf: Configuration):
        headers = {
            'Authorization': f'Bearer {conf.token}',
            'Content-Type': 'application/json'
        }
        
        self.transport = RequestsHTTPTransport(url=f"{conf.hostname}/graphql", headers=headers)
        self.__client = Client(transport=self.transport, fetch_schema_from_transport=True)

    def send_request(self, graphql_query: str, graphql_variables: str = None):
        if graphql_variables is None:
            return self.client.execute(gql(graphql_query))
        else:
            return self.client.execute(gql(graphql_query), json.loads(graphql_variables))



from wikijspy.api_client import ApiClient
from wikijspy.configuration import Configuration
from wikijspy.api.pages import PagesApi
from wikijspy.api.users import UsersApi
from wikijspy.api.authentication import AuthenticationApi
from wikijspy.api.assets import AssetsApi
from wikijspy.types.page_types import *
from wikijspy.types.users_types import *
from wikijspy.types.authentication_types import *
from wikijspy.types.assets_types import *
from wikijspy.types.general import *




class Configuration:
    def __init__(self, hostname: str, api_token: str):
        self.hostname = hostname
        self.token = api_token
    
    @property
    def hostname(self) -> str:
        return self.__hostname
    
    @hostname.setter
    def hostname(self, value: str):
        self.__hostname = value
    
    @property
    def token(self) -> str:
        return self.__token
    
    @token.setter
    def token(self, value: str):
        self.__token = value



from typing import Dict, List, Tuple
from wikijspy.api_client import ApiClient
from wikijspy.api.shared import _generate_output_str
from wikijspy.types.page_types import PageOrderBy, PageOrderByDirection, PageListItemOutput, PageOutput, PageResponseOutput
from wikijspy.types.general import DefaultResponseOutput, ResponseStatusOutput
import re
import json

class PagesApi:
    def __init__(self, api_client: ApiClient) -> None:
        self.api_client = api_client

    def list(self,
             output: PageListItemOutput,
             authorId: int = None,
             creatorId: int = None,
             limit: int = None,
             locale: str = None,
             orderBy: PageOrderBy = None,
             orderByDirection: PageOrderByDirection = None,
             tags: List[str] = None
             ):
        query = """
        query($authorId: Int, $creatorId: Int, $limit: Int, $locale: String, $orderBy: PageOrderBy, $orderByDirection: PageOrderByDirection, $tags: [String!]){
            pages {
                list(
                    authorId: $authorId,
                    creatorId: $creatorId,
                    limit: $limit,
                    locale: $locale,
                    orderBy: $orderBy,
                    orderByDirection: $orderByDirection,
                    tags: $tags
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))
        
        return self.api_client.send_request(query, json.dumps({
                "authorId": authorId,
                "creatorId": creatorId,
                "limit": limit,
                "locale": locale,
                "orderBy": orderBy,
                "orderByDirection": orderByDirection,
                "tags": tags
        }))
    
    def single(self,
        output: PageOutput,
        id: int
    ):
        query = """
        query($id: Int!){
            pages{
                single(
                    id: $id
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))
        
        return self.api_client.send_request(query, json.dumps({
            "id": id
        }))
    
    def create(self,
        output: PageResponseOutput,
        content: str,
        description: str,
        editor: str,
        isPublished: bool,
        isPrivate: bool,
        locale: str,
        path: str,
        tags: List[str],
        title: str,
        publishEndDate: str = None,
        publishStartDate: str = None,
        scriptCss: str = None,
        scriptJs: str = None,
    ):
        query = """
        mutation($content: String!, $description: String!, $editor: String!, $isPublished: Boolean!, $isPrivate: Boolean!, $locale: String!, $path: String!, $publishEndDate: Date, $publishStartDate: Date, $scriptCss: String, $scriptJs: String, $tags: [String]!, $title: String!){
            pages{
                create(
                    content: $content,
                    description: $description,
                    editor: $editor,
                    isPublished: $isPublished,
                    isPrivate: $isPrivate,
                    locale: $locale,
                    path: $path,
                    publishEndDate: $publishEndDate,
                    publishStartDate: $publishStartDate,
                    scriptCss: $scriptCss,
                    scriptJs: $scriptJs,
                    tags: $tags,
                    title: $title
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))
        
        return self.api_client.send_request(query, json.dumps({
            "content": content,
            "description": description,
            "editor": editor,
            "isPublished": isPublished,
            "isPrivate": isPrivate,
            "locale": locale,
            "path": path,
            "tags": tags,
            "title": title,
            "publishEndDate": publishEndDate,
            "publishStartDate": publishStartDate,
            "scriptCss": scriptCss,
            "scriptJs": scriptJs
        }))
        
    def update(self,
        output: PageResponseOutput,
        id: int,
        content: str = None,
        description: str = None,
        editor: str = None,
        isPrivate: bool = None,
        isPublished: bool = None,
        locale: str = None,
        path: str = None,
        publishEndDate: str = None,
        publishStartDate: str = None,
        scriptCss: str = None,
        scriptJs: str = None,
        tags: List[str] = [],
        title: str = None
    ):
        query = """
        mutation($id: Int!, $content: String, $description: String, $editor: String, $isPrivate: Boolean, $isPublished: Boolean, $locale: String, $path: String, $publishEndDate: Date, $publishStartDate: Date, $scriptCss: String, $scriptJs: String, $tags: [String], $title: String) {
            pages {
                update(
                    id: $id
                    content: $content
                    description: $description
                    editor: $editor
                    isPrivate: $isPrivate
                    isPublished: $isPublished
                    locale: $locale
                    path: $path
                    publishEndDate: $publishEndDate
                    publishStartDate: $publishStartDate
                    scriptCss: $scriptCss
                    scriptJs: $scriptJs
                    tags: $tags
                    title: $title
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))
        
        if content is None:
            content = self.single(PageOutput(["content"]), id)["pages"]["single"]["content"]
        
        split_query = query.split("\n")
        
        for key,val in locals().items():
            if val is None:
                split_query[0] = re.sub(r"\$.+, ", "", split_query[0])
                for index,line in enumerate(split_query):
                    if not re.search(f" +{key}: \$\w+", line) is None:
                        split_query[index] = ""
        
        all_vars = {
            "id": id,
            "content": content,
            "description": description,
            "editor": editor,
            "isPrivate": isPrivate,
            "isPublished": isPublished,
            "locale": locale,
            "path": path,
            "publishEndDate": publishEndDate,
            "publishStartDate": publishStartDate,
            "scriptCss": scriptCss,
            "scriptJs": scriptJs,
            "tags": tags,
            "title": title
        }
        
        query_variables = {}
        
        for key,val in all_vars.items():
            if not val is None:
                query_variables[key] = val
                
        return self.api_client.send_request(query, json.dumps(query_variables))
    
    def delete(self,
        output: DefaultResponseOutput,
        id: int
    ):
        query = """
        mutation($id: Int!){
            pages{
                delete(
                    id: $id
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        return self.api_client.send_request(query, json.dumps({
            "id": id
        }))
    
    def render(self,
        output: DefaultResponseOutput,
        id: int
    ):
        query = """
        mutation($id: Int!){
            pages{
                render(
                    id: $id
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))
        
        return self.api_client.send_request(query, json.dumps({
            "id": id
        }))



from enum import Enum
from typing import Any, Dict, List, Tuple
from wikijspy.types.error import InvalidOutputError
from wikijspy.types.general import ResponseStatusOutput, Output, DictOutput

class PageOrderBy(Enum):
    CREATED = 0
    ID = 1
    PATH = 2
    TITLE = 3
    UPDATED = 4

class PageOrderByDirection(Enum):
    ASC = 0
    DESC = 1

class PageListItemOutput(Output):
    _validation_list = [
        "id",
        "path",
        "locale",
        "title",
        "description",
        "contentType",
        "isPublished",
        "isPrivate",
        "privateNS",
        "createdAt",
        "updatedAt",
        "tags"
    ]


class PageOutput(Output):
    _validation_list = [
        "id",
        "path",
        "hash",
        "title",
        "description",
        "isPrivate",
        "isPublished",
        "privateNS",
        "publishStartDate",
        "publishEndDate",
        "tags",
        "content",
        "render",
        "toc",
        "contentType",
        "createdAt",
        "updatedAt",
        "editor",
        "locale",
        "scriptCss",
        "scriptJs",
        "authorId",
        "authorName",
        "authorEmail",
        "creatorId",
        "creatorName",
        "creatorEmail"
    ]


class PageResponseOutput(DictOutput):
    _validation_list = {
        "responseResult": ResponseStatusOutput._validation_list,
        "page": PageOutput._validation_list
    }



from setuptools import setup, find_packages

from os.path import dirname

with open(f"{dirname(__file__)}/requirements.txt", "r") as f:
    install_requires = f.read().splitlines()

setup(
    name='wikijspy',
    version='0.1.5',
    author='WERTBON1789',
    author_email='jan210404@gmx.de',
    packages=find_packages(),
    license='LICENSE',
    description='A simple wikijs API wrapper',
    long_description="""Github Repository: https://github.com/WERTBON1789/wikijs-api-wrapper""",
    install_requires=install_requires
)



import json
from typing import List
from wikijspy.api_client import ApiClient
from wikijspy.types.general import DefaultResponseOutput
from wikijspy.types.users_types import UserMinimalOutput, UserOutput, UserResponseOutput
from wikijspy.api.shared import _generate_output_str

class UsersApi:
    def __init__(self, api_client: ApiClient) -> None:
        self.api_client = api_client
    
    def list(self, output: UserMinimalOutput, filter: str = None, orderBy: str = None):
        query = """
        query($filter: String, $orderBy: String){
            users{
                list(
                    filter: $filter,
                    orderBy: $orderBy
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        return self.api_client.send_request(query, json.dumps({
            "filter": filter,
            "orderBy": orderBy
        }))
        
    def search(self, output: UserMinimalOutput, query: str):
        gql_query = """
        query($query: String!){
            users{
                search(
                    query: $query
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))
        
        return self.api_client.send_request(gql_query, json.dumps({
            "query": query
        }))
    def single(self, output: UserOutput, id: int):
        query = """
        query($id: Int!){
            users{
                single(
                    id: $id 
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        return self.api_client.send_request(query, json.dumps({
            "id": id
        }))
    def profile(self, output):
        query = """
        query{
            users{
                profile{
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        return self.api_client.send_request(query)
    def lastLogins(self, output):
        query = """
        query{
            users{
                lastLogins{
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        return self.api_client.send_request(query)
    def create(self, output: UserResponseOutput, email: str, name: str, providerKey: str, passwordRaw: str = None, groups: List[int] = [], mustChangePassword: bool = False, sendWelcomeEmail: bool = False):
        query = """
        mutation($email: String!, $name: String!, $passwordRaw: String, $providerKey: String!, $groups: [Int]!, $mustChangePassword: Boolean, $sendWelcomeEmail: Boolean){
            users{
                create(
                    email: $email
                    name: $name
                    passwordRaw: $passwordRaw
                    providerKey: $providerKey
                    groups: $groups
                    mustChangePassword: $mustChangePassword
                    sendWelcomeEmail: $sendWelcomeEmail
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        return self.api_client.send_request(query, json.dumps({
            "email": email,
            "name": name,
            "providerKey": providerKey,
            "passwordRaw": passwordRaw,
            "groups": groups,
            "mustChangePassword": mustChangePassword,
            "sendWelcomeEmail": sendWelcomeEmail
        }))
    def update(self, output: DefaultResponseOutput, id: int, email: str = None, name: str = None, newPassword: str = None, groups: List[int] = None, location: str = None, jobTitle: str = None, timezone: str = None, dateFormat: str = None, appearance: str = None):
        query = """
        mutation($id: Int!, $email: String, $name: String, $newPassword: String, $groups: [Int], $location: String, $jobTitle: String, $timezone: String, $dateFormat: String, $appearance: String){
            users{
                update(
                    id: $id
                    email: $email
                    name: $name
                    newPassword: $newPassword
                    groups: $groups
                    location: $location
                    jobTitle: $jobTitle
                    timezone: $timezone
                    dateFormat: $dateFormat
                    appearance: $appearance
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        return self.api_client.send_request(query, json.dumps({
            "id": id,
            "email": email,
            "name": name,
            "newPassword": newPassword,
            "groups": groups,
            "location": location,
            "jobTitle": jobTitle,
            "timezone": timezone,
            "dateFormat": dateFormat,
            "appearance": appearance
        }))
    def delete(self, output: DefaultResponseOutput, id: int, replaceId: int):
        query = """
        mutation($id: Int!, $replaceId: Int!){
            users{
                delete(
                    id: $id
                    replaceId: $replaceId
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        return self.api_client.send_request(query, json.dumps({
            "id": id,
            "replaceId": replaceId
        }))
    def verify(self, output: DefaultResponseOutput, id: int):
        query = """
        
        """.replace('OUTPUT', _generate_output_str(output))

        return self.api_client.send_request(query, json.dumps({
            "id": id
        }))
    def activate(self, output: DefaultResponseOutput, id: int):
        query = """
        mutation($id: Int!){
            users{
                activate(
                    id: $id
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        return self.api_client.send_request(query, json.dumps({
            "id": id
        }))
    def deactivate(self, output: DefaultResponseOutput, id: int):
        query = """
        mutation($id: Int!){
            users{
                deactivate(
                    id: $id
                ){
                    OUTPUT
                }
            }
        }
        """.replace('OUTPUT', _generate_output_str(output))

        return self.api_client.send_request(query, json.dumps({
            "id": id
        }))
    def enableTFA(self, output: DefaultResponseOutput, id: int):
        query = """
        
        """.replace('OUTPUT', _generate_output_str(output))
    def disableTFA(self, output: DefaultResponseOutput, id: int):
        query = """
        
        """.replace('OUTPUT', _generate_output_str(output))
    def resetPassword(self, output: DefaultResponseOutput, id: int):
        query = """
        
        """.replace('OUTPUT', _generate_output_str(output))
    def updateProfile(self, output):
        pass
    def changePassword(self, output):
        pass
